@@ -57,13 +57,13 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <assert.h>
 
 #ifdef HAVE_OPENSSL
 #include <openssl/hmac.h>
 #endif
 
 #include <OpenIPMI/ipmi_msgbits.h>
-#include <OpenIPMI/ipmi_auth.h>
 #include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/ipmi_mc.h>
 #include <OpenIPMI/ipmi_lan.h>
@@ -248,7 +248,7 @@ raw_send(lanserv_data_t *lan,
 			  "Raw LAN send to:");
 	for (i = 0; i < vecs; i++)
 	    len += vec[i].iov_len;
-	slen = snprintf(&dummy, 1, format);
+	slen = snprintf(&dummy, 0, format);
 	slen += len * 3 + 3;
 	str = malloc(slen);
 	if (!str)
@@ -273,7 +273,7 @@ raw_send(lanserv_data_t *lan,
 static void
 return_rmcpp_rsp(lanserv_data_t *lan, session_t *session, msg_t *msg,
 		 unsigned int payload, unsigned char *data, unsigned int len,
-		 unsigned char iana[3], unsigned int payload_id)
+		 unsigned char *iana, unsigned int payload_id)
 {
     uint8_t d[IPMI_LAN_MAX_HEADER_SIZE+IPMI_LAN_MAX_HEADER_SIZE
 	      +IPMI_LAN_MAX_TRAILER_SIZE+1];
@@ -296,8 +296,6 @@ return_rmcpp_rsp(lanserv_data_t *lan, session_t *session, msg_t *msg,
 
     if (payload == 0) {
 	/* Add the IPMI header - fixme -cheap hack */
-	if (hdr_left < 6)
-	    return;
 	hdr_left -= 6;
 	pos -= 6;
 	dlen += 6; /* Adding header, increase total length */
@@ -385,6 +383,7 @@ return_rmcpp_rsp(lanserv_data_t *lan, session_t *session, msg_t *msg,
 
     tpos = pos + 6;
     if (payload == 2) {
+	assert(iana);
 	memcpy(tpos, iana, 3);
 	tpos[3] = 0;
 	ipmi_set_uint16(tpos+4, payload_id);
@@ -511,7 +510,7 @@ lan_return_rsp(channel_t *chan, msg_t *msg, rsp_msg_t *rsp)
 
     return_rsp(lan, msg, NULL, rsp);
 
-    msg = ipmi_mc_get_next_recv_q(chan);
+    msg = lan->sysinfo->mc_get_next_recv_q(chan);
     if (!msg)
 	return;
     while (msg) {
@@ -530,7 +529,7 @@ lan_return_rsp(channel_t *chan, msg_t *msg, rsp_msg_t *rsp)
 
 	chan->free(chan, msg);
 
-	msg = ipmi_mc_get_next_recv_q(chan);
+	msg = lan->sysinfo->mc_get_next_recv_q(chan);
     }
     if (chan->recv_in_q)
 	chan->recv_in_q(chan, 0);
@@ -728,7 +727,7 @@ handle_get_channel_cipher_suites(lanserv_data_t *lan, msg_t *msg)
     if (chan == 0xe)
 	chan = lan->channel.channel_num;
 
-    channels = ipmi_mc_get_channelset(lan->channel.mc);
+    channels = lan->sysinfo->mc_get_channelset(lan->channel.mc);
     channel = channels[chan];
     if (!channel) {
 	return_err(lan, msg, NULL, IPMI_NOT_PRESENT_CC);
@@ -881,6 +880,12 @@ handle_temp_session(lanserv_data_t *lan, msg_t *msg)
     }
 
     auth = msg->data[0] & 0xf;
+    if (auth >= MAX_IPMI_AUTHS) {
+	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
+		 "Activate session failed: Invalid auth: 0x%x", auth);
+	return;
+    }
+
     user = &(lan->users[user_idx]);
     if (! (user->valid)) {
 	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
@@ -1244,8 +1249,7 @@ write_lan_config(lanserv_data_t *lan)
     if (lan->persist_changed) {
 	persist_t *p;
 
-	p = alloc_persist("lanparm.mc%2.2x.%d",
-			  ipmi_mc_get_ipmb(lan->channel.mc),
+	p = alloc_persist("lanparm.mc%2.2x.%d", 0x20,
 			  lan->channel.channel_num);
 	if (!p)
 	    return;
@@ -3034,6 +3038,11 @@ ipmi_handle_lan_msg(lanserv_data_t *lan,
     }
 
     msg.authtype = data[4];
+    if (msg.authtype >= MAX_IPMI_AUTHS) {
+	lan->sysinfo->log(lan->sysinfo, LAN_ERR, &msg,
+		 "LAN msg failure: Invalid authtype");
+	return;
+    }
     msg.data = data+5;
     msg.len = len - 5;
     msg.channel = lan->channel.channel_num;
@@ -3079,7 +3088,7 @@ read_lan_config(lanserv_data_t *lan)
     unsigned int len;
     long iv;
 
-    p = read_persist("lanparm.mc%2.2x.%d", ipmi_mc_get_ipmb(lan->channel.mc),
+    p = read_persist("lanparm.mc%2.2x.%d", 0x20,
 		     lan->channel.channel_num);
 
     if (p && !read_persist_data(p, &data, &len, "max_priv_for_cipher")) {
@@ -3143,7 +3152,7 @@ get_associated_mc(channel_t *chan, uint32_t session_id, unsigned int payload)
     lanserv_data_t *lan = chan->chan_info;
     session_t *session = sid_to_session(lan, session_id);
 
-    if (payload >= LANSERV_NUM_CLOSERS)
+    if (payload >= LANSERV_NUM_CLOSERS || session == NULL)
 	return NULL;
 
     return session->closers[payload].mc;
@@ -3202,7 +3211,7 @@ ipmi_lan_init(lanserv_data_t *lan)
 
     lan->tick_handler.handler = ipmi_lan_tick;
     lan->tick_handler.info = lan;
-    ipmi_register_tick_handler(&lan->tick_handler);
+    lan->sysinfo->register_tick_handler(&lan->tick_handler);
 
  out:
     return rv;
